@@ -6,43 +6,60 @@
 # @Filename: targetdb.py
 # @License: BSD 3-clause (http://www.opensource.org/licenses/BSD-3-Clause)
 
-from peewee import (AutoField, BigIntegerField, BooleanField, DeferredThroughModel,
-                    DoubleField, FloatField, ForeignKeyField, IntegerField,
-                    ManyToManyField, SmallIntegerField, TextField)
+from peewee import (AutoField, BigIntegerField, BooleanField,
+                    DeferredThroughModel, DoubleField, FloatField,
+                    ForeignKeyField, IntegerField, ManyToManyField,
+                    SmallIntegerField, TextField)
 from playhouse.postgres_ext import ArrayField
 
-from . import SDSS5dbModel, catalogdb, database  # noqa
+from .. import BaseModel
+from . import catalogdb, database  # noqa
+
+
+class TargetdbBase(BaseModel):
+
+    class Meta:
+        schema = 'targetdb'
+        database = database
 
 
 AssignmentDeferred = DeferredThroughModel()
 ProgramToTargetDeferred = DeferredThroughModel()
 
 
-class Cadence(SDSS5dbModel):
+class Version(TargetdbBase):
+    label = TextField(null=True)
+    pk = AutoField()
+    target_selection = BooleanField()
+    robostrategy = BooleanField()
+
+    class Meta:
+        table_name = 'version'
+
+
+class Cadence(TargetdbBase):
     delta = ArrayField(field_class=FloatField, null=True)
     delta_max = ArrayField(field_class=FloatField, null=True)
     delta_min = ArrayField(field_class=FloatField, null=True)
     instrument_pk = ArrayField(field_class=IntegerField, null=True)
-    label = TextField(null=True)
+    label = TextField(null=False)
     nexposures = SmallIntegerField(null=True)
     pk = AutoField()
     skybrightness = ArrayField(field_class=FloatField, null=True)
 
     class Meta:
         table_name = 'cadence'
-        schema = 'targetdb'
 
 
-class Observatory(SDSS5dbModel):
+class Observatory(TargetdbBase):
     label = TextField()
     pk = AutoField()
 
     class Meta:
         table_name = 'observatory'
-        schema = 'targetdb'
 
 
-class Field(SDSS5dbModel):
+class Field(TargetdbBase):
     cadence = ForeignKeyField(column_name='cadence_pk',
                               field='pk',
                               model=Cadence,
@@ -54,14 +71,15 @@ class Field(SDSS5dbModel):
                                   null=True)
     pk = AutoField()
     racen = DoubleField(null=True)
-    version = TextField(null=True)
+    version = ForeignKeyField(column_name='version_pk',
+                              field='pk',
+                              model=Version)
 
     class Meta:
         table_name = 'field'
-        schema = 'targetdb'
 
 
-class Design(SDSS5dbModel):
+class Design(TargetdbBase):
     field = ForeignKeyField(column_name='field_pk',
                             field='pk',
                             model=Field,
@@ -71,28 +89,25 @@ class Design(SDSS5dbModel):
 
     class Meta:
         table_name = 'design'
-        schema = 'targetdb'
 
 
-class Instrument(SDSS5dbModel):
+class Instrument(TargetdbBase):
     label = TextField(null=True)
     pk = AutoField()
 
     class Meta:
         table_name = 'instrument'
-        schema = 'targetdb'
 
 
-class PositionerStatus(SDSS5dbModel):
+class PositionerStatus(TargetdbBase):
     label = TextField(null=True)
     pk = AutoField()
 
     class Meta:
         table_name = 'positioner_status'
-        schema = 'targetdb'
 
 
-class PositionerInfo(SDSS5dbModel):
+class PositionerInfo(TargetdbBase):
     apogee = BooleanField(null=False)
     boss = BooleanField(null=False)
     fiducial = BooleanField(null=False)
@@ -100,10 +115,9 @@ class PositionerInfo(SDSS5dbModel):
 
     class Meta:
         table_name = 'positioner_info'
-        schema = 'targetdb'
 
 
-class Positioner(SDSS5dbModel):
+class Positioner(TargetdbBase):
     id = IntegerField(null=True)
     observatory = ForeignKeyField(column_name='observatory_pk',
                                   field='pk',
@@ -120,10 +134,9 @@ class Positioner(SDSS5dbModel):
 
     class Meta:
         table_name = 'positioner'
-        schema = 'targetdb'
 
 
-class Magnitude(SDSS5dbModel):
+class Magnitude(TargetdbBase):
     bp = FloatField(null=True)
     g = FloatField(null=True)
     h = FloatField(null=True)
@@ -134,55 +147,46 @@ class Magnitude(SDSS5dbModel):
 
     class Meta:
         table_name = 'magnitude'
-        schema = 'targetdb'
 
 
-class Version(SDSS5dbModel):
-    label = TextField(null=True)
-    pk = AutoField()
-
-    class Meta:
-        table_name = 'version'
-        schema = 'targetdb'
-
-
-class Category(SDSS5dbModel):
+class Category(TargetdbBase):
     label = TextField(null=True)
     pk = AutoField()
 
     class Meta:
         table_name = 'category'
-        schema = 'targetdb'
 
 
-class Survey(SDSS5dbModel):
+class Survey(TargetdbBase):
     label = TextField(null=True)
     pk = AutoField()
 
     class Meta:
         table_name = 'survey'
-        schema = 'targetdb'
 
 
-class Program(SDSS5dbModel):
+class Program(TargetdbBase):
     category = ForeignKeyField(column_name='category_pk',
                                field='pk',
-                               model=Category,
-                               null=True)
-    label = TextField(null=True)
+                               model=Category)
+    label = TextField()
     pk = AutoField()
     survey = ForeignKeyField(column_name='survey_pk',
                              field='pk',
-                             model=Survey,
-                             null=True)
+                             model=Survey)
+    version = ForeignKeyField(column_name='version_pk',
+                              field='pk',
+                              model=Version)
 
     class Meta:
         table_name = 'program'
-        schema = 'targetdb'
 
 
-class Target(SDSS5dbModel):
-    catalogid = BigIntegerField(null=True)
+class Target(TargetdbBase):
+    catalog = ForeignKeyField(column_name='catalogid',
+                              object_id_name='catalogid',
+                              model=catalogdb.Catalog,
+                              field='catalogid')
     dec = DoubleField(null=True)
     epoch = FloatField(null=True)
     magnitude = ForeignKeyField(column_name='magnitude_pk',
@@ -203,22 +207,16 @@ class Target(SDSS5dbModel):
     instruments = ManyToManyField(Instrument,
                                   through_model=AssignmentDeferred,
                                   backref='targets')
-    programs = ManyToManyField(Program,
-                               through_model=ProgramToTargetDeferred,
-                               backref='targets')
     cadences = ManyToManyField(Cadence,
                                through_model=ProgramToTargetDeferred,
                                backref='targets')
-    versions = ManyToManyField(Version,
-                               through_model=ProgramToTargetDeferred,
-                               backref='targets')
+    parallax = FloatField(null=True)
 
     class Meta:
         table_name = 'target'
-        schema = 'targetdb'
 
 
-class Assignment(SDSS5dbModel):
+class Assignment(TargetdbBase):
     design = ForeignKeyField(Design,
                              column_name='design_pk',
                              field='pk')
@@ -235,10 +233,9 @@ class Assignment(SDSS5dbModel):
 
     class Meta:
         table_name = 'assignment'
-        schema = 'targetdb'
 
 
-class ProgramToTarget(SDSS5dbModel):
+class ProgramToTarget(TargetdbBase):
     cadence = ForeignKeyField(Cadence,
                               column_name='cadence_pk',
                               field='pk')
@@ -251,15 +248,9 @@ class ProgramToTarget(SDSS5dbModel):
                              column_name='target_pk',
                              field='pk',
                              on_delete='CASCADE')
-    version = ForeignKeyField(Version,
-                              column_name='version_pk',
-                              field='pk',
-                              null=True,
-                              on_delete='CASCADE')
 
     class Meta:
         table_name = 'program_to_target'
-        schema = 'targetdb'
 
 
 AssignmentDeferred.set_model(Assignment)
